import React, { useEffect, useMemo, useRef, useState } from 'react'
import styles from './index.less'
import { Table, Tooltip, message, Button, Pagination, Popconfirm, Space, Input } from 'antd'
import { SearchOutlined } from '@ant-design/icons'
import SearchGroup from './components/SearchGroup'
import TablePack from './components/TablePack'
import {
  GetAccountConfigInfo,
  GetAccountPageList,
  getStationListByUserID,
  GetTableJson,
  GetTableDataInfo,
  EditTableDataInfo,
  SaveTableDataInfo,
  DeleteTableDataInfo,
} from '../../apis/process'
import { isJson, isObject } from '../../utils/index'
import IconPack from '../widgets/IconPack'
import FormRender from '../FormRender'

const getFileInfo = (formJson) => {
  let obj = {}
  let parent = formJson?.properties
  if (isObject(parent)) {
    for (let v in parent) {
      let child = parent[v]?.properties
      if (isObject(child)) {
        for (let s in child) {
          obj[s] = child[s]
        }
      }
    }
  }
  return obj
}

const Account = (props) => {

  const userID = window?.globalConfig?.userInfo?.OID || 1
  const { accountName, readOnly } = props
  const [detailShow, setDetailShow] = useState(false)
  const [params, setParams] = useState({
    userID: userID,
    accountName,
    sortFields: '录入时间',
    direction: 'desc',
    total: 0,
    pageIndex: 1,
    pageSize: 100
  })
  const [loading, setLoading] = useState(false)
  const [dataSource, setDataSource] = useState([])
  const [config, setConfig] = useState({ webShowFieldGroup: '', addFieldGroup: '', formJson: {} })
  const [schemaValues, setSchemaValues] = useState({ formJson: {}, values: [] })
  const [submitLoading, setSubmitLoading] = useState(false)
  const [operation, setOperation] = useState({ id: null, state: '添加' })
  const formRenderRef = useRef()
  const searchInput = useRef(null)

  const handleReset = (clearFilters, confirm) => {
    clearFilters()
    confirm({ closeDropdown: true })
  }

  const getColumnSearchProps = (dataIndex) => ({
    filterDropdown: ({ setSelectedKeys, selectedKeys, confirm, clearFilters, close }) => (
      <div
        style={{ padding: 8 }}
        onKeyDown={(e) => e.stopPropagation()}
      >
        <Input
          ref={searchInput}
          placeholder={`搜索 ${dataIndex}`}
          value={selectedKeys[0]}
          onChange={e => setSelectedKeys(e.target.value ? [e.target.value] : [])}
          onPressEnter={() => confirm({ closeDropdown: true })}
          style={{
            marginBottom: 8,
            display: 'block',
            width: '280px',
          }}
        />
        <Space>
          <Button
            type="primary"
            onClick={() => confirm({ closeDropdown: true })}
            icon={<SearchOutlined />}
            size="small"
            style={{ width: 90 }}
          >
            搜索
          </Button>
          <Button
            onClick={() => handleReset(clearFilters, confirm)}
            size="small"
            style={{ width: 90 }}
          >
            重置
          </Button>
        </Space>
      </div>
    ),
    filterIcon: (filtered) => (
      <SearchOutlined
        style={{
          color: filtered ? '#1890ff' : undefined,
        }}
      />
    ),
  })

  const fileColumns = useMemo(() => {
    const { webShowFieldGroup, formJson } = config
    let showField = webShowFieldGroup ? webShowFieldGroup.split(',').filter(v => v) : []
    let json = getFileInfo(formJson)
    let array = []
    showField.forEach(v => {
      array.push({
        title: json[v]?.title || v,
        dataIndex: v,
        width: 120,
        ellipsis: true,
        sorter: true,
        filterSearch: true,
        ...getColumnSearchProps(v),
        render: (value, r) => {
          let props = { ...json[v], value }
          return (
            <TablePack {...props} />
          )
        }
      })
    })
    return array
  }, [config])

  const btns = useMemo(() => {
    if (readOnly) {
      return [
        {
          title: '详情',
          icon: <IconPack.XiangQing />,
        },
      ]
    }
    return [
      {
        title: '详情',
        icon: <IconPack.XiangQing />,
      },
      {
        title: '编辑',
        icon: <IconPack.BianJi />,
      },
      {
        title: '删除',
        icon: <IconPack.ShanChu />,
      }
    ]
  }, [readOnly])

  const columns = useMemo(() => {
    return [
      {
        title: '序号',
        dataIndex: 'r',
        key: 'r',
        width: 50,
        align: 'center',
        fixed: 'left',
      },
      ...fileColumns,
      {
        title: '操作',
        width: btns.length * 35 + 30,
        align: 'center',
        fixed: 'right',
        render: (r) => {
          return (
            <div style={{ display: 'flex', justifyContent: 'center' }}>
              {
                btns.map(v => {
                  if (v.title === '删除') {
                    return (
                      <Popconfirm
                        key={v.title}
                        title='确定要删除数据吗？'
                        onConfirm={() => btnsClick(v.title, r.ID)}
                        okText='确定'
                        cancelText='取消'
                      >
                        <Button
                          type='link'
                          title='删除'
                          style={{ padding: '0 7px' }}
                          icon={v.icon}
                        />
                      </Popconfirm>
                    )
                  }
                  return (
                    <Button
                      key={v.title}
                      type='link'
                      title={v.title}
                      icon={v.icon}
                      style={{ padding: '0 7px' }}
                      onClick={() => btnsClick(v.title, r.ID)}
                    >
                    </Button>
                  )
                })
              }
            </div>
          )
        }
      }
    ]
  }, [btns, fileColumns])

  const btnsClick = async (type, id) => {
    const { addFieldGroup, editFieldGroup, formJson } = config
    const addField = addFieldGroup ? addFieldGroup.split(',').filter(v => v) : []
    const editField = editFieldGroup ? editFieldGroup.split(',').filter(v => v) : []
    setOperation({ type, id })
    if (type === '添加') {
      saveClick(addField, formJson, [], type)
    }
    if (type === '编辑' || type === '详情') {
      const { code, data, msg } = await GetTableDataInfo({ accountName, id })
      if (code === 0) {
        saveClick(editField, formJson, data, type)
      } else {
        message.error(msg)
      }
    }
    if (type === '删除') {
      const { code, data, msg } = await DeleteTableDataInfo({ accountTable: accountName, ids: id })
      if (code === 0) {
        message.success('删除成功!')
        getDataSource()
      } else {
        message.error(msg)
      }
    }
  }

  const saveClick = (addField, formJson, values, type) => {
    let json = JSON.parse(JSON.stringify(formJson))
    let parent = json?.properties
    if (isObject(parent)) {
      for (let v in parent) {
        let hidden = true
        let child = parent[v]?.properties
        if (isObject(child)) {
          for (let s in child) {
            if (type === '详情') {
              child[s].disabled = true
            }
            if (addField.includes(s)) {
              child[s].hidden = false
              hidden = false
            } else {
              child[s].hidden = true
            }
          }
        }
        parent[v].hidden = hidden
      }
    }
    setSchemaValues({ formJson: json, values })
    setDetailShow(true)
  }

  const back = () => {
    setDetailShow(false)
    getDataSource()
  }

  const submit = async () => {
    const { formValue, relationForm, errors } = await formRenderRef.current.getValues()
    if (errors.length) {
      return message.error('请完善表单内容')
    }
    const { type, id } = operation
    setSubmitLoading(true)
    if (type === '添加') {
      const { code, data } = await SaveTableDataInfo({ accountTable: accountName, id: operation.id, values: formValue })
      if (code === 0) {
        message.success('保存成功!')
        setSubmitLoading(false)
        setDetailShow(false)
        getDataSource()
      }
    }
    if (type === '编辑') {
      const { code, data } = await EditTableDataInfo({ accountTable: accountName, id: operation.id, values: formValue })
      if (code === 0) {
        message.success('修改!')
        setSubmitLoading(false)
        setDetailShow(false)
        getDataSource()
      }
    }
  }

  const search = (values) => {
    setParams({ ...params, ...values })
    getDataSource({ ...values })
  }

  const getConfig = async () => {
    const { code, data } = await GetAccountConfigInfo(accountName)
    const res = await GetTableJson(accountName)
    if (code === 0 && res.code === 0) {
      setConfig({ ...data, formJson: isJson(res.data) ? JSON.parse(res.data) : {} })
    }
  }

  const getDataSource = async (param = {}) => {
    setLoading(true)
    const { code, data, msg } = await GetAccountPageList({ ...params, ...param })
    if (code === 0) {
      setDataSource(isJson(data.jsonData) ? JSON.parse(data.jsonData) : [])
      setParams({ ...params, total: data.totalCount, pageIndex: data.pageIndex, pageSize: data.pageSize })
    } else {
      setDataSource([])
      message.error(msg)
    }
    setLoading(false)
  }

  const pageChange = (pageIndex, pageSize) => {
    getDataSource({ pageIndex, pageSize })
  }

  const getData = () => {
    getConfig()
    getDataSource()
  }

  const tableChange = (page, filters, sorter) => {
    let queryWheres = []
    Object.keys(filters).forEach(k => {
      if (filters[k]?.[0]) {
        queryWheres.push({ field: k, type: '包括', value: filters[k][0] })
      }
    })
    let param = {
      sortFields: sorter.order ? sorter.field : '录入时间',
      direction: sorter.order !== 'ascend' ? 'desc' : 'asc',
      queryWheres
    }
    setParams({ ...params, ...param })
    getDataSource(param)
  }

  useEffect(() => {
    getData()
  }, [])

  return (
    <div className={styles.account}>
      <div className={styles.tableRender} style={{ display: !detailShow ? 'block' : 'none' }}>
        <div className={styles.content}>
          <div className={styles.top}>
            <SearchGroup
              onChange={search}
              btnsClick={btnsClick}
              readOnly={readOnly}
            />
          </div>
          <div className={styles.bottom}>
            <Table
              size='small'
              rowKey='ID'
              bordered
              loading={loading}
              columns={columns}
              dataSource={dataSource}
              pagination={false}
              scroll={{ y: 'calc(100% - 50px)' }}
              onChange={tableChange}
            />
          </div>
        </div>
        <div className={styles.footer}>
          <div className={styles.total}>共计{params.total}条数据</div>
          <div className={styles.pagination}>
            <Pagination
              showQuickJumper
              pageSize={params.pageSize}
              current={params.pageIndex}
              total={params.total}
              onChange={pageChange}
            />
          </div>
        </div>
      </div>
      {
        detailShow ? (
          <div className={styles.tableDetail}>
            <div className={styles.formBox}>
              <FormRender ref={formRenderRef} schemaValues={schemaValues} />
            </div>
            <div className={styles.formBtns}>
              <Button
                onClick={back}
              >
                返回
              </Button>
              <Button
                style={{ marginLeft: '10px', display: ['添加', '编辑'].includes(operation.type) ? 'block' : 'none' }}
                type='primary'
                loading={submitLoading}
                onClick={submit}
              >
                提交
              </Button>
            </div>
          </div>
        ) : null
      }
    </div>
  )

}

export default Account